@@ -764,6 +764,7 @@ TALER_EXCHANGE_refresh_prepare (const struct TALER_CoinSpendPrivateKeyP *melt_pr
   unsigned int i;
   unsigned int j;
   struct GNUNET_HashContext *hash_context;
+  struct TALER_Amount total;
 
   /* build up melt data structure */
   for (i=0;i<TALER_CNC_KAPPA;i++)
@@ -799,10 +800,44 @@ TALER_EXCHANGE_refresh_prepare (const struct TALER_CoinSpendPrivateKeyP *melt_pr
     md.fresh_coins[i] = GNUNET_new_array (fresh_pks_len,
                                           struct FreshCoinP);
     for (j=0;j<fresh_pks_len;j++)
+    {
       setup_fresh_coin (&md.fresh_coins[i][j],
                         &fresh_pks[j]);
+    }
   }
 
+  /* verify that melt_amount is above total cost */
+  GNUNET_assert (GNUNET_OK ==
+		 TALER_amount_get_zero (melt_amount->currency,
+					&total));
+  for (j=0;j<fresh_pks_len;j++)
+  {
+    if ( (GNUNET_OK !=
+	  TALER_amount_add (&total,
+			    &total,
+			    &fresh_pks[j].value)) ||
+	 (GNUNET_OK !=
+	  TALER_amount_add (&total,
+			    &total,
+			    &fresh_pks[j].fee_withdraw)) )
+    {
+      GNUNET_break (0);
+      free_melt_data (&md);
+      return NULL;
+    }
+  }
+  if (1 ==
+      TALER_amount_cmp (&total,
+			melt_amount) )
+  {
+    /* Eh, this operation is more expensive than the 
+       @a melt_amount. This is not OK. */
+    GNUNET_break (0);
+    free_melt_data (&md);
+    return NULL;
+  }
+
+
   /* now compute melt session hash */
   hash_context = GNUNET_CRYPTO_hash_context_start ();
   for (i=0;i<fresh_pks_len;i++)
@@ -851,10 +886,18 @@ TALER_EXCHANGE_refresh_prepare (const struct TALER_CoinSpendPrivateKeyP *melt_pr
       GNUNET_CRYPTO_hash (&coin_pub.eddsa_pub,
                           sizeof (struct GNUNET_CRYPTO_EcdsaPublicKey),
                           &coin_hash);
-      coin_ev_size = GNUNET_CRYPTO_rsa_blind (&coin_hash,
-                                              &fc->blinding_key.bks,
-                                              md.fresh_pks[j].rsa_public_key,
-                                              &coin_ev);
+      if (GNUNET_YES !=
+          GNUNET_CRYPTO_rsa_blind (&coin_hash,
+                                   &fc->blinding_key.bks,
+                                   md.fresh_pks[j].rsa_public_key,
+                                   &coin_ev,
+                                   &coin_ev_size))
+      {
+        GNUNET_break_op (0);
+        GNUNET_CRYPTO_hash_context_abort (hash_context);
+        free_melt_data (&md);
+        return NULL;
+      }
       GNUNET_CRYPTO_hash_context_read (hash_context,
                                        coin_ev,
                                        coin_ev_size);
@@ -951,20 +994,21 @@ struct TALER_EXCHANGE_RefreshMeltHandle
  *
  * @param rmh melt handle
  * @param json json reply with the signature
+ * @param[out] exchange_pub public key of the exchange used for the signature
  * @param[out] noreveal_index set to the noreveal index selected by the exchange
  * @return #GNUNET_OK if the signature is valid, #GNUNET_SYSERR if not
  */
 static int
 verify_refresh_melt_signature_ok (struct TALER_EXCHANGE_RefreshMeltHandle *rmh,
                                   const json_t *json,
+                                  struct TALER_ExchangePublicKeyP *exchange_pub,
                                   uint16_t *noreveal_index)
 {
   struct TALER_ExchangeSignatureP exchange_sig;
-  struct TALER_ExchangePublicKeyP exchange_pub;
   const struct TALER_EXCHANGE_Keys *key_state;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_fixed_auto ("exchange_sig", &exchange_sig),
-    GNUNET_JSON_spec_fixed_auto ("exchange_pub", &exchange_pub),
+    GNUNET_JSON_spec_fixed_auto ("exchange_pub", exchange_pub),
     GNUNET_JSON_spec_uint16 ("noreveal_index", noreveal_index),
     GNUNET_JSON_spec_end()
   };
@@ -983,7 +1027,7 @@ verify_refresh_melt_signature_ok (struct TALER_EXCHANGE_RefreshMeltHandle *rmh,
   key_state = TALER_EXCHANGE_get_keys (rmh->exchange);
   if (GNUNET_OK !=
       TALER_EXCHANGE_test_signing_key (key_state,
-                                   &exchange_pub))
+                                       exchange_pub))
   {
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
@@ -1006,7 +1050,7 @@ verify_refresh_melt_signature_ok (struct TALER_EXCHANGE_RefreshMeltHandle *rmh,
       GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_MELT,
                                   &confirm.purpose,
                                   &exchange_sig.eddsa_signature,
-                                  &exchange_pub.eddsa_pub))
+                                  &exchange_pub->eddsa_pub))
   {
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
@@ -1126,6 +1170,7 @@ handle_refresh_melt_finished (void *cls,
 {
   struct TALER_EXCHANGE_RefreshMeltHandle *rmh = cls;
   uint16_t noreveal_index = TALER_CNC_KAPPA; /* invalid value */
+  struct TALER_ExchangePublicKeyP exchange_pub;
 
   rmh->job = NULL;
   switch (response_code)
@@ -1136,6 +1181,7 @@ handle_refresh_melt_finished (void *cls,
     if (GNUNET_OK !=
         verify_refresh_melt_signature_ok (rmh,
                                           json,
+                                          &exchange_pub,
                                           &noreveal_index))
     {
       GNUNET_break_op (0);
@@ -1146,6 +1192,7 @@ handle_refresh_melt_finished (void *cls,
       rmh->melt_cb (rmh->melt_cb_cls,
                     response_code,
                     noreveal_index,
+                    (0 == response_code) ? NULL : &exchange_pub,
                     json);
       rmh->melt_cb = NULL;
     }
@@ -1190,6 +1237,7 @@ handle_refresh_melt_finished (void *cls,
     rmh->melt_cb (rmh->melt_cb_cls,
                   response_code,
                   UINT16_MAX,
+                  NULL,
                   json);
   TALER_EXCHANGE_refresh_melt_cancel (rmh);
 }
@@ -1304,8 +1352,9 @@ TALER_EXCHANGE_refresh_melt (struct TALER_EXCHANGE_Handle *exchange,
 
     GNUNET_CRYPTO_ecdhe_key_get_public (&mc->transfer_priv[j].ecdhe_priv,
                                         &transfer_pub.ecdhe_pub);
-    json_array_append (transfer_pubs,
-                       GNUNET_JSON_from_data_auto (&transfer_pub));
+    GNUNET_assert (0 ==
+                   json_array_append_new (transfer_pubs,
+                                          GNUNET_JSON_from_data_auto (&transfer_pub)));
   }
 
   /* now secret_encs */
@@ -1322,16 +1371,18 @@ TALER_EXCHANGE_refresh_melt (struct TALER_EXCHANGE_Handle *exchange,
                    TALER_transfer_encrypt (&md->link_secrets[j],
                                            &trans_sec,
                                            &els));
-    json_array_append (secret_encs,
-                       GNUNET_JSON_from_data_auto (&els));
+    GNUNET_assert (0 ==
+                   json_array_append_new (secret_encs,
+                                          GNUNET_JSON_from_data_auto (&els)));
   }
 
   /* now new_denoms */
   for (i=0;i<md->num_fresh_coins;i++)
   {
-    json_array_append (new_denoms,
-                       GNUNET_JSON_from_rsa_public_key
-                       (md->fresh_pks[i].rsa_public_key));
+    GNUNET_assert (0 ==
+                   json_array_append_new (new_denoms,
+                                          GNUNET_JSON_from_rsa_public_key
+                                          (md->fresh_pks[i].rsa_public_key)));
   }
 
   /* now link_encs */
@@ -1349,11 +1400,13 @@ TALER_EXCHANGE_refresh_melt (struct TALER_EXCHANGE_Handle *exchange,
       TALER_refresh_encrypt (&rld,
 			     &md->link_secrets[j],
 			     &rle);
-      json_array_append (tmp,
-                         GNUNET_JSON_from_data_auto (&rle));
+      GNUNET_assert (0 ==
+                     json_array_append_new (tmp,
+                                            GNUNET_JSON_from_data_auto (&rle)));
     }
-    json_array_append (link_encs,
-                       tmp);
+    GNUNET_assert (0 ==
+                   json_array_append_new (link_encs,
+                                          tmp));
   }
 
   /* now coin_evs */
@@ -1373,17 +1426,33 @@ TALER_EXCHANGE_refresh_melt (struct TALER_EXCHANGE_Handle *exchange,
       GNUNET_CRYPTO_hash (&coin_pub.eddsa_pub,
                           sizeof (struct GNUNET_CRYPTO_EcdsaPublicKey),
                           &coin_hash);
-      coin_ev_size = GNUNET_CRYPTO_rsa_blind (&coin_hash,
-                                              &fc->blinding_key.bks,
-                                              md->fresh_pks[i].rsa_public_key,
-                                              &coin_ev);
-      json_array_append (tmp,
-                         GNUNET_JSON_from_data (coin_ev,
-                                                coin_ev_size));
+      if (GNUNET_YES !=
+          GNUNET_CRYPTO_rsa_blind (&coin_hash,
+                                   &fc->blinding_key.bks,
+                                   md->fresh_pks[i].rsa_public_key,
+                                   &coin_ev,
+                                   &coin_ev_size))
+      {
+        /* This should have been noticed during the preparation stage. */
+        GNUNET_break (0);
+        json_decref (new_denoms);
+        json_decref (tmp);
+        json_decref (coin_evs);
+        json_decref (melt_coin);
+        json_decref (transfer_pubs);
+        json_decref (secret_encs);
+        json_decref (link_encs);
+        return NULL;
+      }
+      GNUNET_assert (0 ==
+                     json_array_append_new (tmp,
+                                            GNUNET_JSON_from_data (coin_ev,
+                                                                   coin_ev_size)));
       GNUNET_free (coin_ev);
     }
-    json_array_append (coin_evs,
-                       tmp);
+    GNUNET_assert (0 ==
+                   json_array_append_new (coin_evs,
+                                          tmp));
   }
 
   /* finally, assemble main JSON request from constitutent arrays */
@@ -1529,14 +1598,14 @@ refresh_reveal_ok (struct TALER_EXCHANGE_RefreshRevealHandle *rrh,
 {
   unsigned int i;
   json_t *jsona;
-  struct GNUNET_JSON_Specification spec[] = {
+  struct GNUNET_JSON_Specification outer_spec[] = {
     GNUNET_JSON_spec_json ("ev_sigs", &jsona),
     GNUNET_JSON_spec_end()
   };
 
   if (GNUNET_OK !=
       GNUNET_JSON_parse (json,
-                         spec,
+                         outer_spec,
                          NULL, NULL))
   {
     GNUNET_break_op (0);
@@ -1546,12 +1615,14 @@ refresh_reveal_ok (struct TALER_EXCHANGE_RefreshRevealHandle *rrh,
   {
     /* We expected an array of coins */
     GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (outer_spec);
     return GNUNET_SYSERR;
   }
   if (rrh->md->num_fresh_coins != json_array_size (jsona))
   {
     /* Number of coins generated does not match our expectation */
     GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (outer_spec);
     return GNUNET_SYSERR;
   }
   for (i=0;i<rrh->md->num_fresh_coins;i++)
@@ -1563,7 +1634,6 @@ refresh_reveal_ok (struct TALER_EXCHANGE_RefreshRevealHandle *rrh,
     struct GNUNET_CRYPTO_RsaSignature *sig;
     struct TALER_CoinSpendPublicKeyP coin_pub;
     struct GNUNET_HashCode coin_hash;
-
     struct GNUNET_JSON_Specification spec[] = {
       GNUNET_JSON_spec_rsa_signature ("ev_sig", &blind_sig),
       GNUNET_JSON_spec_end()
@@ -1580,6 +1650,7 @@ refresh_reveal_ok (struct TALER_EXCHANGE_RefreshRevealHandle *rrh,
                            NULL, NULL))
     {
       GNUNET_break_op (0);
+      GNUNET_JSON_parse_free (outer_spec);
       return GNUNET_SYSERR;
     }
 
@@ -1603,11 +1674,13 @@ refresh_reveal_ok (struct TALER_EXCHANGE_RefreshRevealHandle *rrh,
     {
       GNUNET_break_op (0);
       GNUNET_CRYPTO_rsa_signature_free (sig);
+      GNUNET_JSON_parse_free (outer_spec);
       return GNUNET_SYSERR;
     }
     coin_privs[i] = fc->coin_priv;
     sigs[i].rsa_signature = sig;
   }
+  GNUNET_JSON_parse_free (outer_spec);
   return GNUNET_OK;
 }
 
@@ -1731,7 +1804,7 @@ TALER_EXCHANGE_refresh_reveal (struct TALER_EXCHANGE_Handle *exchange,
   struct GNUNET_CURL_Context *ctx;
   struct MeltData *md;
   unsigned int j;
-  
+
   GNUNET_assert (GNUNET_YES ==
 		 MAH_handle_is_ready (exchange));
   md = deserialize_melt_data (refresh_data,
@@ -1761,8 +1834,9 @@ TALER_EXCHANGE_refresh_reveal (struct TALER_EXCHANGE_Handle *exchange,
 	 noreval index! */
       continue;
     }
-    json_array_append (transfer_privs,
-                       GNUNET_JSON_from_data_auto (&md->melted_coin.transfer_priv[j]));
+    GNUNET_assert (0 ==
+                   json_array_append_new (transfer_privs,
+                                          GNUNET_JSON_from_data_auto (&md->melted_coin.transfer_priv[j])));
   }
 
   /* build main JSON request */
