@@ -26,7 +26,7 @@
 #include "taler_json_lib.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <microhttpd.h>
-#include "fakebank.h"
+#include "taler_fakebank_lib.h"
 
 
 /**
@@ -62,7 +62,7 @@ static struct GNUNET_SCHEDULER_Task *timeout_task;
 /**
  * Handle to our fakebank.
  */
-static struct FAKEBANK_Handle *fakebank;
+static struct TALER_FAKEBANK_Handle *fakebank;
 
 /**
  * Result of the testcases, #GNUNET_OK on success
@@ -122,12 +122,12 @@ enum OpCode
   OC_WIRE,
 
   /**
-   * Verify exchange's /wire/deposits method.
+   * Verify exchange's /track/transfer method.
    */
   OC_WIRE_DEPOSITS,
 
   /**
-   * Verify exchange's /deposit/wtid method.
+   * Verify exchange's /track/transaction method.
    */
   OC_DEPOSIT_WTID,
 
@@ -514,14 +514,14 @@ struct Command
     } wire;
 
     /**
-     * Information for the /wire/deposits's command.
+     * Information for the /track/transfer's command.
      */
     struct {
 
       /**
        * Handle to the wire deposits request.
        */
-      struct TALER_EXCHANGE_WireDepositsHandle *wdh;
+      struct TALER_EXCHANGE_TrackTransferHandle *wdh;
 
       /**
        * Reference to a command providing a WTID. If set, we use the
@@ -529,7 +529,7 @@ struct Command
        * #OC_DEPOSIT_WTID or an #OC_CHECK_BANK_TRANSFER.  In the
        * case of the bank transfer, we check that the total amount
        * claimed by the exchange matches the total amount transferred
-       * by the bank.  In the case of a /deposit/wtid, we check
+       * by the bank.  In the case of a /track/transaction, we check
        * that the wire details match.
        */
       const char *wtid_ref;
@@ -552,14 +552,14 @@ struct Command
     } wire_deposits;
 
     /**
-     * Information for the /deposit/wtid command.
+     * Information for the /track/transaction command.
      */
     struct {
 
       /**
        * Handle to the deposit wtid request.
        */
-      struct TALER_EXCHANGE_DepositWtidHandle *dwh;
+      struct TALER_EXCHANGE_TrackTransactionHandle *dwh;
 
       /**
        * Which /deposit operation should we obtain WTID data for?
@@ -585,6 +585,11 @@ struct Command
        */
       struct GNUNET_OS_Process *aggregator_proc;
 
+      /**
+       * ID of task called whenever we get a SIGCHILD.
+       */
+      struct GNUNET_SCHEDULER_Task *child_death_task;
+
     } run_aggregator;
 
     struct {
@@ -681,12 +686,6 @@ struct InterpreterState
  */
 static struct GNUNET_DISK_PipeHandle *sigpipe;
 
-/**
- * ID of task called whenever we get a SIGCHILD.
- */
-static struct GNUNET_SCHEDULER_Task *child_death_task;
-
-
 
 /**
  * The testcase failed, return with an error code.
@@ -1028,12 +1027,14 @@ reserve_withdraw_cb (void *cls,
  * @param cls closure with the interpreter state
  * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful deposit;
  *                    0 if the exchange's reply is bogus (fails to follow the protocol)
+ * @param exchange_pub public key the exchange used for signing
  * @param obj the received JSON reply, should be kept as proof (and, in case of errors,
  *            be forwarded to the customer)
  */
 static void
 deposit_cb (void *cls,
             unsigned int http_status,
+            const struct TALER_ExchangePublicKeyP *exchange_pub,
             const json_t *obj)
 {
   struct InterpreterState *is = cls;
@@ -1062,12 +1063,14 @@ deposit_cb (void *cls,
  *                    0 if the exchange's reply is bogus (fails to follow the protocol)
  * @param noreveal_index choice by the exchange in the cut-and-choose protocol,
  *                    UINT16_MAX on error
+ * @param exchange_pub public key the exchange used for signing
  * @param full_response full response from the exchange (for logging, in case of errors)
  */
 static void
 melt_cb (void *cls,
          unsigned int http_status,
          uint16_t noreveal_index,
+         const struct TALER_ExchangePublicKeyP *exchange_pub,
          const json_t *full_response)
 {
   struct InterpreterState *is = cls;
@@ -1262,7 +1265,7 @@ maint_child_death (void *cls)
   const struct GNUNET_DISK_FileHandle *pr;
   char c[16];
 
-  child_death_task = NULL;
+  cmd->details.run_aggregator.child_death_task = NULL;
   pr = GNUNET_DISK_pipe_handle (sigpipe, GNUNET_DISK_PIPE_END_READ);
   GNUNET_break (0 < GNUNET_DISK_file_read (pr, &c, sizeof (c)));
   GNUNET_OS_process_wait (cmd->details.run_aggregator.aggregator_proc);
@@ -1388,6 +1391,7 @@ wire_cb (void *cls,
  *
  * @param cls closure
  * @param http_status HTTP status code we got, 0 on exchange protocol violation
+ * @param exchange_pub public key the exchange used for signing
  * @param json original json reply (may include signatures, those have then been
  *        validated already)
  * @param wtid extracted wire transfer identifier, or NULL if the exchange could
@@ -1400,11 +1404,12 @@ wire_cb (void *cls,
 static void
 wire_deposits_cb (void *cls,
                   unsigned int http_status,
+                  const struct TALER_ExchangePublicKeyP *exchange_pub,
                   const json_t *json,
                   const struct GNUNET_HashCode *h_wire,
                   const struct TALER_Amount *total_amount,
                   unsigned int details_length,
-                  const struct TALER_WireDepositDetails *details)
+                  const struct TALER_TrackTransferDetails *details)
 {
   struct InterpreterState *is = cls;
   struct Command *cmd = &is->commands[is->ip];
@@ -1515,6 +1520,7 @@ wire_deposits_cb (void *cls,
  *
  * @param cls closure
  * @param http_status HTTP status code we got, 0 on exchange protocol violation
+ * @param exchange_pub public key the exchange used for signing
  * @param json original json reply (may include signatures, those have then been
  *        validated already)
  * @param wtid wire transfer identifier used by the exchange, NULL if exchange did not
@@ -1527,6 +1533,7 @@ wire_deposits_cb (void *cls,
 static void
 deposit_wtid_cb (void *cls,
                  unsigned int http_status,
+                 const struct TALER_ExchangePublicKeyP *exchange_pub,
                  const json_t *json,
                  const struct TALER_WireTransferIdentifierRawP *wtid,
                  struct GNUNET_TIME_Absolute execution_time,
@@ -1580,12 +1587,14 @@ deposit_wtid_cb (void *cls,
  * @param cls closure
  * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful deposit;
  *                    0 if the exchange's reply is bogus (fails to follow the protocol)
+ * @param exchange_pub public key the exchange used for signing @a obj
  * @param obj the received JSON reply, should be kept as proof (and, in particular,
  *            be forwarded to the customer)
  */
 static void
 refund_cb (void *cls,
            unsigned int http_status,
+           const struct TALER_ExchangePublicKeyP *exchange_pub,
            const json_t *obj)
 {
   struct InterpreterState *is = cls;
@@ -1744,6 +1753,7 @@ interpreter_run (void *cls)
     GNUNET_TIME_round_abs (&execution_date);
     cmd->details.admin_add_incoming.aih
       = TALER_EXCHANGE_admin_add_incoming (exchange,
+                                           "http://localhost:18080/",
                                            &reserve_pub,
                                            &amount,
                                            execution_date,
@@ -1906,6 +1916,7 @@ interpreter_run (void *cls)
       }
       TALER_JSON_hash (contract,
                        &h_contract);
+      json_decref (contract);
       wire = json_loads (cmd->details.deposit.wire_details,
                          JSON_REJECT_DUPLICATES,
                          NULL);
@@ -2155,7 +2166,7 @@ interpreter_run (void *cls)
       }
     }
     cmd->details.wire_deposits.wdh
-      = TALER_EXCHANGE_wire_deposits (exchange,
+      = TALER_EXCHANGE_track_transfer (exchange,
                                       &cmd->details.wire_deposits.wtid,
                                       &wire_deposits_cb,
                                       is);
@@ -2193,7 +2204,7 @@ interpreter_run (void *cls)
                        &h_contract);
       json_decref (contract);
       cmd->details.deposit_wtid.dwh
-        = TALER_EXCHANGE_deposit_wtid (exchange,
+        = TALER_EXCHANGE_track_transaction (exchange,
                                        &ref->details.deposit.merchant_priv,
                                        &h_wire,
                                        &h_contract,
@@ -2223,9 +2234,10 @@ interpreter_run (void *cls)
         return;
       }
       pr = GNUNET_DISK_pipe_handle (sigpipe, GNUNET_DISK_PIPE_END_READ);
-      child_death_task = GNUNET_SCHEDULER_add_read_file (GNUNET_TIME_UNIT_FOREVER_REL,
-                                                         pr,
-                                                         &maint_child_death, is);
+      cmd->details.run_aggregator.child_death_task
+        = GNUNET_SCHEDULER_add_read_file (GNUNET_TIME_UNIT_FOREVER_REL,
+                                          pr,
+                                          &maint_child_death, is);
       return;
     }
   case OC_CHECK_BANK_TRANSFER:
@@ -2242,7 +2254,7 @@ interpreter_run (void *cls)
         return;
       }
       if (GNUNET_OK !=
-          FAKEBANK_check (fakebank,
+          TALER_FAKEBANK_check (fakebank,
                           &amount,
                           cmd->details.check_bank_transfer.account_debit,
                           cmd->details.check_bank_transfer.account_credit,
@@ -2258,7 +2270,7 @@ interpreter_run (void *cls)
   case OC_CHECK_BANK_TRANSFERS_EMPTY:
     {
       if (GNUNET_OK !=
-          FAKEBANK_check_empty (fakebank))
+          TALER_FAKEBANK_check_empty (fakebank))
       {
         GNUNET_break (0);
         fail (is);
@@ -2512,7 +2524,7 @@ do_shutdown (void *cls)
                     "Command %u (%s) did not complete\n",
                     i,
                     cmd->label);
-        TALER_EXCHANGE_wire_deposits_cancel (cmd->details.wire_deposits.wdh);
+        TALER_EXCHANGE_track_transfer_cancel (cmd->details.wire_deposits.wdh);
         cmd->details.wire_deposits.wdh = NULL;
       }
       break;
@@ -2523,7 +2535,7 @@ do_shutdown (void *cls)
                     "Command %u (%s) did not complete\n",
                     i,
                     cmd->label);
-        TALER_EXCHANGE_deposit_wtid_cancel (cmd->details.deposit_wtid.dwh);
+        TALER_EXCHANGE_track_transaction_cancel (cmd->details.deposit_wtid.dwh);
         cmd->details.deposit_wtid.dwh = NULL;
       }
       break;
@@ -2537,6 +2549,11 @@ do_shutdown (void *cls)
         GNUNET_OS_process_destroy (cmd->details.run_aggregator.aggregator_proc);
         cmd->details.run_aggregator.aggregator_proc = NULL;
       }
+      if (NULL != cmd->details.run_aggregator.child_death_task)
+      {
+        GNUNET_SCHEDULER_cancel (cmd->details.run_aggregator.child_death_task);
+        cmd->details.run_aggregator.child_death_task = NULL;
+      }
       break;
     case OC_CHECK_BANK_TRANSFER:
       break;
@@ -2570,7 +2587,7 @@ do_shutdown (void *cls)
   GNUNET_free (is);
   if (NULL != fakebank)
   {
-    FAKEBANK_stop (fakebank);
+    TALER_FAKEBANK_stop (fakebank);
     fakebank = NULL;
   }
   if (NULL != exchange)
@@ -3002,7 +3019,7 @@ run (void *cls)
                           &rc);
   GNUNET_assert (NULL != ctx);
   rc = GNUNET_CURL_gnunet_rc_create (ctx);
-  fakebank = FAKEBANK_start (8082);
+  fakebank = TALER_FAKEBANK_start (8082);
   exchange = TALER_EXCHANGE_connect (ctx,
                                      "http://localhost:8081",
                                      &cert_cb, is,
