@@ -432,12 +432,11 @@ deposit_cb (void *cls,
     return GNUNET_SYSERR;
   }
   au->row_id = row_id;
-  au->wire = (json_t *) wire;
+  au->wire = json_incref ((json_t *) wire);
   au->execution_time = GNUNET_TIME_absolute_get ();
   (void) GNUNET_TIME_round_abs (&au->execution_time);
   TALER_JSON_hash (au->wire,
                    &au->h_wire);
-  json_incref (au->wire);
   GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_NONCE,
                               &au->wtid,
                               sizeof (au->wtid));
@@ -757,6 +756,7 @@ run_aggregation (void *cls)
                 "Preparing wire transfer of %s to %s\n",
                 amount_s,
                 TALER_B2S (&au->merchant_pub));
+    GNUNET_free (amount_s);
   }
   au->wp = wp;
   au->ph = wp->wire_plugin->prepare_wire_transfer (wp->wire_plugin->cls,
@@ -773,8 +773,8 @@ run_aggregation (void *cls)
     GNUNET_free_non_null (au->additional_rows);
     if (NULL != au->wire)
       json_decref (au->wire);
-    au = NULL;
     GNUNET_free (au);
+    au = NULL;
     /* start again */
     task = GNUNET_SCHEDULER_add_now (&run_aggregation,
                                      NULL);
@@ -809,7 +809,10 @@ prepare_cb (void *cls,
   struct TALER_EXCHANGEDB_Session *session = au->session;
 
   if (NULL != au->wire)
+  {
     json_decref (au->wire);
+    au->wire = NULL;
+  }
   GNUNET_free_non_null (au->additional_rows);
   if (NULL == buf)
   {
@@ -993,6 +996,7 @@ run_transfers (void *cls)
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to obtain database session!\n");
     global_ret = GNUNET_SYSERR;
+    GNUNET_SCHEDULER_shutdown ();
     return;
   }
   if (GNUNET_OK !=
@@ -1094,7 +1098,11 @@ main (int argc,
                           gettext_noop ("background process that aggregates and executes wire transfers to merchants"),
                           options,
                           &run, NULL))
+  {
+    GNUNET_free ((void*) argv);
     return 1;
+  }
+  GNUNET_free ((void*) argv);
   return global_ret;
 }
 
