@@ -411,7 +411,7 @@ TMH_DB_execute_refund (struct MHD_Connection *connection,
                           session);
     TMH_plugin->free_coin_transaction_list (TMH_plugin->cls,
                                             tl);
-    return TMH_RESPONSE_reply_deposit_unknown (connection);
+    return TMH_RESPONSE_reply_transaction_unknown (connection);
   }
   /* handle if conflicting refund found */
   if (GNUNET_SYSERR == refund_found)
@@ -1185,6 +1185,7 @@ check_commitment (struct MHD_Connection *connection,
   struct TALER_TransferPublicKeyP transfer_pub_check;
   struct TALER_EXCHANGEDB_RefreshCommitCoin *commit_coins;
   unsigned int j;
+  int ret;
 
   if (GNUNET_OK !=
       TMH_plugin->get_refresh_commit_link (TMH_plugin->cls,
@@ -1262,18 +1263,19 @@ check_commitment (struct MHD_Connection *connection,
     GNUNET_CRYPTO_hash (&coin_pub,
                         sizeof (struct TALER_CoinSpendPublicKeyP),
                         &h_msg);
-    if (0 == (buf_len =
-              GNUNET_CRYPTO_rsa_blind (&h_msg,
-                                       &link_data.blinding_key.bks,
-                                       denom_pubs[j].rsa_public_key,
-                                       &buf)))
+    if (GNUNET_YES !=
+        GNUNET_CRYPTO_rsa_blind (&h_msg,
+                                 &link_data.blinding_key.bks,
+                                 denom_pubs[j].rsa_public_key,
+                                 &buf,
+                                 &buf_len))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "blind failed\n");
-      GNUNET_free (commit_coins);
-      return (MHD_YES == TMH_RESPONSE_reply_internal_error (connection,
+                  "Blind failed (bad denomination key!?)\n");
+      ret = (MHD_YES == TMH_RESPONSE_reply_internal_error (connection,
                                                             "Blinding error"))
           ? GNUNET_NO : GNUNET_SYSERR;
+      goto cleanup;
     }
 
     if ( (buf_len != commit_coins[j].coin_ev_size) ||
@@ -1285,20 +1287,25 @@ check_commitment (struct MHD_Connection *connection,
                   "blind envelope does not match for k=%u, old=%d\n",
                   off,
                   (int) j);
-      GNUNET_free (commit_coins);
-      return send_melt_commitment_error (connection,
-                                         session,
-                                         session_hash,
-                                         melt,
-                                         off,
-                                         j,
-                                         "envelope");
+      GNUNET_free (buf);
+      ret = send_melt_commitment_error (connection,
+                                        session,
+                                        session_hash,
+                                        melt,
+                                        off,
+                                        j,
+                                        "envelope");
+      goto cleanup;
     }
     GNUNET_free (buf);
   }
-  GNUNET_free (commit_coins);
+  ret = GNUNET_OK;
 
-  return GNUNET_OK;
+ cleanup:
+  for (j = 0; j < num_newcoins; j++)
+    GNUNET_free (commit_coins[j].coin_ev);
+  GNUNET_free (commit_coins);
+  return ret;
 }
 
 
@@ -1389,6 +1396,7 @@ execute_refresh_reveal_transaction (struct MHD_Connection *connection,
 {
   unsigned int j;
   struct TMH_KS_StateHandle *key_state;
+  int ret;
 
   START_TRANSACTION (session, connection);
   if (GNUNET_OK !=
@@ -1415,15 +1423,17 @@ execute_refresh_reveal_transaction (struct MHD_Connection *connection,
                                           j);
     if (NULL == ev_sigs[j].rsa_signature)
     {
-      TMH_KS_release (key_state);
-      return TMH_RESPONSE_reply_internal_db_error (connection);
+      ret = TMH_RESPONSE_reply_internal_db_error (connection);
+      goto cleanup;
     }
   }
-  TMH_KS_release (key_state);
   COMMIT_TRANSACTION (session, connection);
-  return TMH_RESPONSE_reply_refresh_reveal_success (connection,
-                                                    refresh_session->num_newcoins,
-                                                    ev_sigs);
+  ret = TMH_RESPONSE_reply_refresh_reveal_success (connection,
+                                                   refresh_session->num_newcoins,
+                                                   ev_sigs);
+ cleanup:
+  TMH_KS_release (key_state);
+  return ret;
 }
 
 
@@ -1525,8 +1535,11 @@ TMH_DB_execute_refresh_reveal (struct MHD_Connection *connection,
                                             ev_sigs,
                                             commit_coins);
   for (i=0;i<refresh_session.num_newcoins;i++)
+  {
     if (NULL != ev_sigs[i].rsa_signature)
-    GNUNET_CRYPTO_rsa_signature_free (ev_sigs[i].rsa_signature);
+      GNUNET_CRYPTO_rsa_signature_free (ev_sigs[i].rsa_signature);
+    GNUNET_free (commit_coins[i].coin_ev);
+  }
   for (j=0;j<refresh_session.num_newcoins;j++)
     if (NULL != denom_pubs[j].rsa_public_key)
       GNUNET_CRYPTO_rsa_public_key_free (denom_pubs[j].rsa_public_key);
@@ -1761,12 +1774,12 @@ struct WtidTransactionContext
   /**
    * Head of DLL with details for /wire/deposit response.
    */
-  struct TMH_WireDepositDetail *wdd_head;
+  struct TMH_TrackTransferDetail *wdd_head;
 
   /**
    * Head of DLL with details for /wire/deposit response.
    */
-  struct TMH_WireDepositDetail *wdd_tail;
+  struct TMH_TrackTransferDetail *wdd_tail;
 
   /**
    * JSON array with details about the individual deposits.
@@ -1810,7 +1823,7 @@ handle_transaction_data (void *cls,
 {
   struct WtidTransactionContext *ctx = cls;
   struct TALER_Amount delta;
-  struct TMH_WireDepositDetail *wdd;
+  struct TMH_TrackTransferDetail *wdd;
 
   if (GNUNET_SYSERR == ctx->is_valid)
     return;
@@ -1861,7 +1874,7 @@ handle_transaction_data (void *cls,
       return;
     }
   }
-  wdd = GNUNET_new (struct TMH_WireDepositDetail);
+  wdd = GNUNET_new (struct TMH_TrackTransferDetail);
   wdd->deposit_value = *deposit_value;
   wdd->deposit_fee = *deposit_fee;
   wdd->h_contract = *h_contract;
@@ -1874,7 +1887,7 @@ handle_transaction_data (void *cls,
 
 
 /**
- * Execute a "/wire/deposits".  Returns the transaction information
+ * Execute a "/track/transfer".  Returns the transaction information
  * associated with the given wire transfer identifier.
  *
  * @param connection the MHD connection to handle
@@ -1882,13 +1895,13 @@ handle_transaction_data (void *cls,
  * @return MHD result code
  */
 int
-TMH_DB_execute_wire_deposits (struct MHD_Connection *connection,
-                             const struct TALER_WireTransferIdentifierRawP *wtid)
+TMH_DB_execute_track_transfer (struct MHD_Connection *connection,
+                               const struct TALER_WireTransferIdentifierRawP *wtid)
 {
   int ret;
   struct WtidTransactionContext ctx;
   struct TALER_EXCHANGEDB_Session *session;
-  struct TMH_WireDepositDetail *wdd;
+  struct TMH_TrackTransferDetail *wdd;
 
   if (NULL == (session = TMH_plugin->get_session (TMH_plugin->cls)))
   {
@@ -1921,11 +1934,11 @@ TMH_DB_execute_wire_deposits (struct MHD_Connection *connection,
                                           "wtid");
     goto cleanup;
   }
-  ret = TMH_RESPONSE_reply_wire_deposit_details (connection,
-                                                 &ctx.total,
-                                                 &ctx.merchant_pub,
-                                                 &ctx.h_wire,
-                                                 ctx.wdd_head);
+  ret = TMH_RESPONSE_reply_track_transfer_details (connection,
+                                                   &ctx.total,
+                                                   &ctx.merchant_pub,
+                                                   &ctx.h_wire,
+                                                   ctx.wdd_head);
  cleanup:
   while (NULL != (wdd = ctx.wdd_head))
   {
@@ -2003,8 +2016,8 @@ handle_wtid_data (void *cls,
 
   if (NULL == wtid)
   {
-    ctx->res = TMH_RESPONSE_reply_deposit_pending (ctx->connection,
-                                                   execution_time);
+    ctx->res = TMH_RESPONSE_reply_transfer_pending (ctx->connection,
+                                                    execution_time);
   }
   else
   {
@@ -2018,21 +2031,21 @@ handle_wtid_data (void *cls,
     }
     else
     {
-      ctx->res = TMH_RESPONSE_reply_deposit_wtid (ctx->connection,
-                                                  &ctx->h_contract,
-                                                  &ctx->h_wire,
-                                                  &ctx->coin_pub,
-                                                  &coin_delta,
-                                                  ctx->transaction_id,
-                                                  wtid,
-                                                  execution_time);
+      ctx->res = TMH_RESPONSE_reply_track_transaction (ctx->connection,
+                                                       &ctx->h_contract,
+                                                       &ctx->h_wire,
+                                                       &ctx->coin_pub,
+                                                       &coin_delta,
+                                                       ctx->transaction_id,
+                                                       wtid,
+                                                       execution_time);
     }
   }
 }
 
 
 /**
- * Execute a "/deposit/wtid".  Returns the transfer information
+ * Execute a "/track/transaction".  Returns the transfer information
  * associated with the given deposit.
  *
  * @param connection the MHD connection to handle
@@ -2044,12 +2057,12 @@ handle_wtid_data (void *cls,
  * @return MHD result code
  */
 int
-TMH_DB_execute_deposit_wtid (struct MHD_Connection *connection,
-                             const struct GNUNET_HashCode *h_contract,
-			     const struct GNUNET_HashCode *h_wire,
-			     const struct TALER_CoinSpendPublicKeyP *coin_pub,
-			     const struct TALER_MerchantPublicKeyP *merchant_pub,
-			     uint64_t transaction_id)
+TMH_DB_execute_track_transaction (struct MHD_Connection *connection,
+                                  const struct GNUNET_HashCode *h_contract,
+                                  const struct GNUNET_HashCode *h_wire,
+                                  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+                                  const struct TALER_MerchantPublicKeyP *merchant_pub,
+                                  uint64_t transaction_id)
 {
   int ret;
   struct DepositWtidContext ctx;
@@ -2084,7 +2097,7 @@ TMH_DB_execute_deposit_wtid (struct MHD_Connection *connection,
   if (GNUNET_NO == ret)
   {
     GNUNET_break (GNUNET_SYSERR == ctx.res);
-    return TMH_RESPONSE_reply_deposit_unknown (connection);
+    return TMH_RESPONSE_reply_transaction_unknown (connection);
   }
   if (GNUNET_SYSERR == ctx.res)
   {
