@@ -561,7 +561,6 @@ TMH_REFRESH_handler_refresh_melt (struct TMH_RequestHandler *rh,
   json_t *link_encs;
   json_t *transfer_pubs;
   json_t *secret_encs;
-  json_t *coin_detail;
   int res;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_json ("new_denoms", &new_denoms),
@@ -605,14 +604,6 @@ TMH_REFRESH_handler_refresh_melt (struct TMH_RequestHandler *rh,
     return TMH_RESPONSE_reply_arg_invalid (connection,
                                            "transfer_pubs");
   }
-  coin_detail = json_array_get (coin_evs, 0);
-  if (NULL == coin_detail)
-  {
-    // FIXME: generate proper HTTP response!
-    GNUNET_break_op (0);
-    GNUNET_JSON_parse_free (spec);
-    return (GNUNET_SYSERR == res) ? MHD_NO : MHD_YES;
-  }
   res = handle_refresh_melt_json (connection,
                                   new_denoms,
                                   melt_coin,
@@ -649,12 +640,11 @@ handle_refresh_reveal_json (struct MHD_Connection *connection,
   res = GNUNET_OK;
   for (i = 0; i < TALER_CNC_KAPPA - 1; i++)
   {
-    if (GNUNET_OK != res)
-      break;
     struct GNUNET_JSON_Specification tp_spec[] = {
       GNUNET_JSON_spec_fixed_auto (NULL, &transfer_privs[i]),
       GNUNET_JSON_spec_end ()
     };
+
     if (GNUNET_OK != res)
       break;
     res = TMH_PARSE_json_array (connection,
@@ -698,7 +688,6 @@ TMH_REFRESH_handler_refresh_reveal (struct TMH_RequestHandler *rh,
 {
   struct GNUNET_HashCode session_hash;
   int res;
-  json_t *reveal_detail;
   json_t *root;
   json_t *transfer_privs;
   struct GNUNET_JSON_Specification spec[] = {
@@ -735,14 +724,6 @@ TMH_REFRESH_handler_refresh_reveal (struct TMH_RequestHandler *rh,
     return TMH_RESPONSE_reply_arg_invalid (connection,
                                            "transfer_privs");
   }
-  reveal_detail = json_array_get (transfer_privs, 0);
-  if (NULL == reveal_detail)
-  {
-    // FIXME: generate proper HTTP response!
-    GNUNET_JSON_parse_free (spec);
-    GNUNET_break_op (0);
-    return (GNUNET_SYSERR == res) ? MHD_NO : MHD_YES;
-  }
   res = handle_refresh_reveal_json (connection,
                                     &session_hash,
                                     transfer_privs);
