@@ -416,12 +416,14 @@ struct TALER_EXCHANGE_DepositHandle;
  * @param cls closure
  * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful deposit;
  *                    0 if the exchange's reply is bogus (fails to follow the protocol)
+ * @param sign_key exchange key used to sign @a obj, or NULL
  * @param obj the received JSON reply, should be kept as proof (and, in case of errors,
  *            be forwarded to the customer)
  */
 typedef void
 (*TALER_EXCHANGE_DepositResultCallback) (void *cls,
                                          unsigned int http_status,
+                                         const struct TALER_ExchangePublicKeyP *sign_key,
                                          const json_t *obj);
 
 
@@ -502,12 +504,14 @@ struct TALER_EXCHANGE_RefundHandle;
  * @param cls closure
  * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful deposit;
  *                    0 if the exchange's reply is bogus (fails to follow the protocol)
+ * @param sign_key exchange key used to sign @a obj, or NULL
  * @param obj the received JSON reply, should be kept as proof (and, in particular,
  *            be forwarded to the customer)
  */
 typedef void
 (*TALER_EXCHANGE_RefundResultCallback) (void *cls,
 					unsigned int http_status,
+                                        const struct TALER_ExchangePublicKeyP *sign_key,
 					const json_t *obj);
 
 
@@ -833,12 +837,14 @@ struct TALER_EXCHANGE_RefreshMeltHandle;
  *                    0 if the exchange's reply is bogus (fails to follow the protocol)
  * @param noreveal_index choice by the exchange in the cut-and-choose protocol,
  *                    UINT16_MAX on error
+ * @param sign_key exchange key used to sign @a full_response, or NULL
  * @param full_response full response from the exchange (for logging, in case of errors)
  */
 typedef void
 (*TALER_EXCHANGE_RefreshMeltCallback) (void *cls,
                                        unsigned int http_status,
                                        uint16_t noreveal_index,
+                                       const struct TALER_ExchangePublicKeyP *sign_key,
                                        const json_t *full_response);
 
 
@@ -1053,6 +1059,7 @@ typedef void
  * to the operators of the exchange.
  *
  * @param exchange the exchange handle; the exchange must be ready to operate
+ * @param admin_url URL of the administrative interface of the exchange
  * @param reserve_pub public key of the reserve
  * @param amount amount that was deposited
  * @param execution_date when did we receive the amount
@@ -1068,6 +1075,7 @@ typedef void
  */
 struct TALER_EXCHANGE_AdminAddIncomingHandle *
 TALER_EXCHANGE_admin_add_incoming (struct TALER_EXCHANGE_Handle *exchange,
+                                   const char *admin_url,
                                    const struct TALER_ReservePublicKeyP *reserve_pub,
                                    const struct TALER_Amount *amount,
                                    struct GNUNET_TIME_Absolute execution_date,
@@ -1087,46 +1095,12 @@ void
 TALER_EXCHANGE_admin_add_incoming_cancel (struct TALER_EXCHANGE_AdminAddIncomingHandle *aai);
 
 
-/* ********************* /wire/deposits *********************** */
+/* ********************* /track/transfer *********************** */
 
 /**
- * @brief A /wire/deposits Handle
+ * @brief A /track/transfer Handle
  */
-struct TALER_EXCHANGE_WireDepositsHandle;
-
-
-/**
- * Details for one of the /deposit operations that the
- * exchange combined into a single wire transfer.
- */
-struct TALER_WireDepositDetails
-{
-  /**
-   * Hash of the contract.
-   */
-  struct GNUNET_HashCode h_contract;
-
-  /**
-   * Which coin was deposited?
-   */
-  struct TALER_CoinSpendPublicKeyP coin_pub;
-
-  /**
-   * Value of the deposit (including fee).
-   */
-  struct TALER_Amount coin_value;
-
-  /**
-   * Fee charged by the exchange for the deposit.
-   */
-  struct TALER_Amount coin_fee;
-
-  /**
-   * Merchant's transaction identifier.
-   */
-  uint64_t transaction_id;
-
-};
+struct TALER_EXCHANGE_TrackTransferHandle;
 
 
 /**
@@ -1135,23 +1109,24 @@ struct TALER_WireDepositDetails
  *
  * @param cls closure
  * @param http_status HTTP status code we got, 0 on exchange protocol violation
+ * @param sign_key exchange key used to sign @a json, or NULL
  * @param json original json reply (may include signatures, those have then been
  *        validated already)
- * @param wtid extracted wire transfer identifier, or NULL if the exchange could
- *             not provide any (set only if @a http_status is #MHD_HTTP_OK)
+ * @param h_wire hash of the wire transfer address the transfer went to, or NULL on error
  * @param total_amount total amount of the wire transfer, or NULL if the exchange could
  *             not provide any @a wtid (set only if @a http_status is #MHD_HTTP_OK)
  * @param details_length length of the @a details array
  * @param details array with details about the combined transactions
  */
 typedef void
-(*TALER_EXCHANGE_WireDepositsCallback)(void *cls,
-                                       unsigned int http_status,
-                                       const json_t *json,
-                                       const struct GNUNET_HashCode *h_wire,
-                                       const struct TALER_Amount *total_amount,
-                                       unsigned int details_length,
-                                       const struct TALER_WireDepositDetails *details);
+(*TALER_EXCHANGE_TrackTransferCallback)(void *cls,
+                                        unsigned int http_status,
+                                        const struct TALER_ExchangePublicKeyP *sign_key,
+                                        const json_t *json,
+                                        const struct GNUNET_HashCode *h_wire,
+                                        const struct TALER_Amount *total_amount,
+                                        unsigned int details_length,
+                                        const struct TALER_TrackTransferDetails *details);
 
 
 /**
@@ -1164,11 +1139,11 @@ typedef void
  * @param cb_cls closure for @a cb
  * @return handle to cancel operation
  */
-struct TALER_EXCHANGE_WireDepositsHandle *
-TALER_EXCHANGE_wire_deposits (struct TALER_EXCHANGE_Handle *exchange,
-                              const struct TALER_WireTransferIdentifierRawP *wtid,
-                              TALER_EXCHANGE_WireDepositsCallback cb,
-                              void *cb_cls);
+struct TALER_EXCHANGE_TrackTransferHandle *
+TALER_EXCHANGE_track_transfer (struct TALER_EXCHANGE_Handle *exchange,
+                               const struct TALER_WireTransferIdentifierRawP *wtid,
+                               TALER_EXCHANGE_TrackTransferCallback cb,
+                               void *cb_cls);
 
 
 /**
@@ -1178,16 +1153,16 @@ TALER_EXCHANGE_wire_deposits (struct TALER_EXCHANGE_Handle *exchange,
  * @param wdh the wire deposits request handle
  */
 void
-TALER_EXCHANGE_wire_deposits_cancel (struct TALER_EXCHANGE_WireDepositsHandle *wdh);
+TALER_EXCHANGE_track_transfer_cancel (struct TALER_EXCHANGE_TrackTransferHandle *wdh);
 
 
-/* ********************* /deposit/wtid *********************** */
+/* ********************* /track/transaction *********************** */
 
 
 /**
- * @brief A /deposit/wtid Handle
+ * @brief A /track/transaction Handle
  */
-struct TALER_EXCHANGE_DepositWtidHandle;
+struct TALER_EXCHANGE_TrackTransactionHandle;
 
 
 /**
@@ -1195,6 +1170,7 @@ struct TALER_EXCHANGE_DepositWtidHandle;
  *
  * @param cls closure
  * @param http_status HTTP status code we got, 0 on exchange protocol violation
+ * @param sign_key exchange key used to sign @a json, or NULL
  * @param json original json reply (may include signatures, those have then been
  *        validated already)
  * @param wtid wire transfer identifier used by the exchange, NULL if exchange did not
@@ -1203,16 +1179,17 @@ struct TALER_EXCHANGE_DepositWtidHandle;
  * @param coin_contribution contribution to the @a total_amount of the deposited coin (may be NULL)
  */
 typedef void
-(*TALER_EXCHANGE_DepositWtidCallback)(void *cls,
-                                      unsigned int http_status,
-                                      const json_t *json,
-                                      const struct TALER_WireTransferIdentifierRawP *wtid,
-                                      struct GNUNET_TIME_Absolute execution_time,
-                                      const struct TALER_Amount *coin_contribution);
+(*TALER_EXCHANGE_TrackTransactionCallback)(void *cls,
+                                           unsigned int http_status,
+                                           const struct TALER_ExchangePublicKeyP *sign_key,
+                                           const json_t *json,
+                                           const struct TALER_WireTransferIdentifierRawP *wtid,
+                                           struct GNUNET_TIME_Absolute execution_time,
+                                           const struct TALER_Amount *coin_contribution);
 
 
 /**
- * Obtain the wire transfer details for a given deposit.
+ * Obtain the wire transfer details for a given transaction.
  *
  * @param exchange the exchange to query
  * @param merchant_priv the merchant's private key
@@ -1224,15 +1201,15 @@ typedef void
  * @param cb_cls closure for @a cb
  * @return handle to abort request
  */
-struct TALER_EXCHANGE_DepositWtidHandle *
-TALER_EXCHANGE_deposit_wtid (struct TALER_EXCHANGE_Handle *exchange,
-                             const struct TALER_MerchantPrivateKeyP *merchant_priv,
-                             const struct GNUNET_HashCode *h_wire,
-                             const struct GNUNET_HashCode *h_contract,
-                             const struct TALER_CoinSpendPublicKeyP *coin_pub,
-                             uint64_t transaction_id,
-                             TALER_EXCHANGE_DepositWtidCallback cb,
-                             void *cb_cls);
+struct TALER_EXCHANGE_TrackTransactionHandle *
+TALER_EXCHANGE_track_transaction (struct TALER_EXCHANGE_Handle *exchange,
+                                  const struct TALER_MerchantPrivateKeyP *merchant_priv,
+                                  const struct GNUNET_HashCode *h_wire,
+                                  const struct GNUNET_HashCode *h_contract,
+                                  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+                                  uint64_t transaction_id,
+                                  TALER_EXCHANGE_TrackTransactionCallback cb,
+                                  void *cb_cls);
 
 
 /**
@@ -1242,7 +1219,7 @@ TALER_EXCHANGE_deposit_wtid (struct TALER_EXCHANGE_Handle *exchange,
  * @param dwh the wire deposits request handle
  */
 void
-TALER_EXCHANGE_deposit_wtid_cancel (struct TALER_EXCHANGE_DepositWtidHandle *dwh);
+TALER_EXCHANGE_track_transaction_cancel (struct TALER_EXCHANGE_TrackTransactionHandle *dwh);
 
 
 
